/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
##*************************************
##
## E.S.O. - VLT project
##
## "@(#) $Id: minrtsFrameData.h,v 1.25 2004/04/16 12:45:44 jaffe Exp $" 
##
## minrtsFrameData.h"
## 
## who      when     what
##-------- -------- ------------------
##jdejong 04/17/02 created for cmm
##
*/
/* $Id: minrtsFrameData.h,v 1.25 2004/04/16 12:45:44 jaffe Exp $ */
#ifndef _MINRTSFRAMEDATA_H
#define _MINRTSFRAMEDATA_H
/*----------------------------------------------------------------------- */
/* Structure and methods for MIDI frame data                              */
/* This is related to imaging_data_row structures                         */
/*----------------------------------------------------------------------- */

#include "imaging_detector.h"
#include "imaging_data.h"
#include "imaging_fdata.h"
#include <math.h>

typedef struct      /* structure for minrtsFrameData */
{
  /**
   * a label
   */
  char   *title;
  /**
   * central time of exposure (sec)
   */
  double time;  
  /**
   * Primary delay
   */
  double opd[2];   
  /**
   * Secondary delay
   */
  double local_opd[2];
  /**
   * exposure length
   */
  float  exp;
  /**
   * position offset from source position
   */
  float  offset[2];
  /**
   * rotation angle on sky
   */
  float  rotation;
  /**
   * frame number
   */
  long   frame;
  /**
   * source id
   */
  int    reference;
  /**
   * step number in observing cycle
   */
  int    step;
  /**
   * number of regions of data
   */
  int    nregion;
  /**
   * Number of columns (1st dim.) per region
   */
  int    *nx;
  /**
   * Number of rows (2nd dim.) per region
   */
  int    *ny;

  /**
   *  x coordinate lower-left corner 
   */
  int *x0;
  
  /**
   * y coordinate lower-left corner
   */
  int *y0;

  /**
   * optical train numbers contributing to the measurment
   */
  int    *opt_train;
  /**
   * data type 'T'=target, 'R'=reference, 'S'=sky, 'B'=background
   */
  char   *targtype;
  /**
   * target number for each region
   */
  int    *target;
  /**
   * weight for example total number of frames accumlated
   */
  float   weight;
  /**
   * mean value per region
   */
  float  *mean;
  /**
   * rms of values per region
   */
  float  *rms;
  /**
   * array of data arrays, one per region as 1D
   */
  float  **data;
} minrtsFrameData;

/**
 *   Make an minrtsFrameData structure from an imaging_data_table_def.
 *   Use minrtsFrameDataKill to destroy the result when not needed.
 *   Inputs:
 *    \param   detect     pointer to imaging_data table structure
 *   Returns:
 *      An minrtsFrameData structure pointer ; NULL indicates failure
 */
minrtsFrameData* minrtsFrameDataMake (imaging_detector_table_def* detect);

/**
 *   Delete an minrtsFrameData structure.
 *   Inputs:
 *     \param  data    pointer to the minrtsFrameData structure
 */
void minrtsFrameDataKill (minrtsFrameData *data);

/***************************************************************
 * Read through all raw files reading separate target and sky frames 
 * and accumulating them in given structures; 
 * Normalize by number of frames.
 * If there are no offsource measurements, offData is zero filled.
 * Inputs:
 *    fileList     array of file names of raw MIDI data
 *    nFiles       number of file names in fileList
 *    nDrop        drop this many points after change of chop phase
 *    onFData      FrameData to receive average on-source data
 *    offFData     FrameData to receive average off-source data
 *    ierr         error code, 0=>OK
 **************************************************************** */
void minrtsFrameDataUnChop(char **fileList, int nFiles, int nDrop,
			   minrtsFrameData *onFData, minrtsFrameData *offFData,
			   int *ierr);
/**
 *   Average a set of Frames in the form of imaging_data_table_rows
 *   producing a  minrtsFrameData.
 *   Descriptive information is copied from the first frame.
 *   Use minrtsFrameDataKill to destroy the result when not needed.
 *   Inputs:
 *   \param    detect    structure describing detector layout
 *   \param    nframes   Number of frames to be averaged
 *   \param    frames    Frame data to be averaged
 *   Returns:
 *   An minrtsFrameData pointer; NULL indicates failure
 */
minrtsFrameData* minrtsFrameDataAverage (imaging_detector_table_def* detect,
					 int nframes, imaging_data_row** frames);

/**
 *   Get statistics for each region and fill in mean and rms entries.
 *   Inputs:
 *    \param   data      minrtsFrameData to operate on
 */
void minrtsFrameDataStats (minrtsFrameData* data);

/**
 *   Convert minrtsFrameData to an image of the detector.
 *   Inputs
 *    \param   data      minrtsFrameData to operate on
 *   Output:
 *     \param  nx        Number of columns in result
 *     \param  ny        Number of rows in result
 *   Returns:
 *      A 2D array of pixel values; NULL indicates failure
 *      zeroes are used for pixels not in a region
 */
float* minrtsFrameDataImage (minrtsFrameData* data, 
			      int *nx, int *ny);

/**
 *   Pixel by pixel addition of data row to frame
 *   Inputs:
 *    \param   data      existing frame data object
 *    \param   row       new data image row
 */
void minrtsFrameDataRowAdd (minrtsFrameData* data,
			imaging_data_row* frames);

/**
 *   Pixel by pixel out = in1 + in2
 *   Inputs:
 *    \param   in1       first minrtsFrameData operand
 *    \param   in2       second minrtsFrameData operand
 *   Output:
 *    \param   out       output minrtsFrameData (must exist prior to call)
 */
void minrtsFrameDataAdd (minrtsFrameData* out,
			 minrtsFrameData* in1,
			 minrtsFrameData* in2);

/**
 *   Pixel by pixel out = in1 - in2
 *   Inputs:
 *    \param   in1       first minrtsFrameData operand
 *    \param   in2       second minrtsFrameData operand
 *   Output:
 *    \param   out       output minrtsFrameData (must exist prior to call)
 */
void minrtsFrameDataSub (minrtsFrameData* out,
			 minrtsFrameData* in1,
			 minrtsFrameData* in2);

/**
 *   Pixel by pixel out = in1 / in2
 *   Inputs:
 *    \param   in1       first minrtsFrameData operand
 *    \param   in2       second minrtsFrameData operand
 *   Output:
 *    \param   out       output minrtsFrameData (must exist prior to call)
 */
void minrtsFrameDataDiv (minrtsFrameData* out,
			 minrtsFrameData* in1,
			 minrtsFrameData* in2);

/**
 *   Pixel by pixel out = in1 / in2
 *   Inputs:
 *    \param   in1       first minrtsFrameData operand
 *     \param  in2       second minrtsFrameData operand
 *   Output:
 *     \param  out       output minrtsFrameData (must exist prior to call)
 */
void minrtsFrameDataMul (minrtsFrameData* out,
			 minrtsFrameData* in1,
			 minrtsFrameData* in2);
/**
 * multiply by scalar constant
 * \param  factor   float
 */
void minrtsFrameDataScalarMul (minrtsFrameData* in, float factor);

/**
 * add a scalar constant
 * \param  add   float
 */
void minrtsFrameDataScalarAdd (minrtsFrameData* in, float add);

/**
 * take sqrt; <= 0 -> 0
 */

void minrtsFrameDataSqrt (minrtsFrameData* in);

/**
 * reduce mean^2 and mean to rms
 * \param  out    rms
 * \param  in     mean^2
 * \param  out    mean
 */
void minrtsFrameDataRMS (minrtsFrameData* out, 
			 minrtsFrameData* in1,
			 minrtsFrameData* in2);
   /**  
     * duplicate an existing FrameData structure
     * Inputs:
     * \param   in     pointer to minrtsFrameData   target of insertion
     *  Outputs:       pointer to minrtsFrameData   data from specified region
   */
minrtsFrameData *minrtsFrameDataClone (minrtsFrameData *in);
/**
  *    Write the contents of a minrtsFrameData structure out
  *    into a FITS table data on disk, setting many values
  *    to elementary defaults, but at least also copying
  *    the detector structure
  *    Inputs:
  *    \param  in          minrtsFrameData pointer  where data comes from
  *    \param fileName     char string              name of output FITS file
  *    \param ierr         pointer to int           if not 0 its not good
  *    Outputs:  none
  */
void minrtsFrameDataWriteFits (minrtsFrameData* in, char* fileName, 
   int *ierr) ;

/**
  *    Write the contents of an array of minrtsFrameData structures 
  *    into a FITS table data on disk, setting many values
  *    to elementary defaults, but at least also copying
  *    the detector structure
  *    Inputs:
  *    \param  in          array ofminrtsFrameData pointers  data 
  *    \param  nFrame      int   number of frames
  *    \param fileName     char string              name of output FITS file
  *    \param ierr         pointer to int           if not 0 its not good
  *    Outputs:  none
  */
void minrtsFrameDataWriteFitsArray (minrtsFrameData **in, int nFrame,
   char* fileName, int *ierr) ;


/**
  *    Read the contents of a minrtsFrameData structure 
  *    from a FITS table data on disk
  *    Inputs:
  *    \param fileName     char string              name of output FITS file
  *    \param ierr         pointer to int           if not 0 its not good
  *    Outputs:  frame     pointer to minrtsFrameData
  */
minrtsFrameData *minrtsFrameDataReadFits (char* fileName, int *ierr) ;
minrtsFrameData *minrtsFrameDataReadFitsRow (char* fileName, 
   int row, int *ierr) ;

/**
  *    compute pixel by pixel mean and rms
  *    from a FITS table data on disk
  *    Inputs:
  *    \param fileName     char string              name of input FITS file
  *    \param mean         pointer to pointer to FrameData  output mean
  *    \param rms         pointer to pointer to FrameData  output rms
  *    \param ierr         pointer to int           if not 0 its not good
  */
void minrtsFrameDataFitsMeanRms (char *fileName, minrtsFrameData **mean, 
   minrtsFrameData **rms, int *ierr);

/**  
 * store external float data into one of the
 * regions of a FrameData structure
 * Inputs:
 * \param   in     pointer to minrtsFrameData   target of insertion
 * \param   region int                          which region to modify, (1-rel)
 * \param   nx     int                          x-size of input data
 * \param   ny     int                          y-size of input data
 * \param   data   pointer to float             data to insert
 * \param   ierr   pointer to int               if not 0, screwed up
 */
void minrtsFrameDataSetData (minrtsFrameData* in, int region, 
   int nx, int ny, float* data, int *ierr) ;

/**  
 * get float data from one of the
 * regions of a FrameData structure
 * Inputs:
 * \param   in     pointer to minrtsFrameData   target of insertion
 * \param   region int                          which region to modify, (1-rel)
 * \param   ierr   pointer to int               if not 0, screwed up
 */
float * minrtsFrameDataGetData (minrtsFrameData* in, int region, int *ierr);


/**  
  * optionally multiply each region by a weight mask
  * and then sum all the pixels, to either in 0 or 1 dim
  * Inputs:
  * \param   in     pointer to minrtsFrameData   input frames
  * \param   mask   pointer to minrtsFrameData   mask frame
     can be NULL (no masking)
  * \param   nDim   int                          #dimension in result
  *  Outputs:       pointer to minrtsFrameData   compressed data 
  */
minrtsFrameData * minrtsFrameDataCompress (minrtsFrameData *in, 
     minrtsFrameData *mask, int nDim); 

/**  
  * Blank pixels where mask is zero or less.
  * Inputs:
  * \param   in     pointer to minrtsFrameData   input frame
  * \param   mask   pointer to minrtsFrameData   mask frame
  * \param   blank  blanking value
  */
void minrtsFrameDataBlank (minrtsFrameData *in, minrtsFrameData *mask, 
			   float blank); 

/**  
  * Checks for saturation, return 0 if OK, 1=>bad
  * Since the A/D units wrap on saturation there is no easy test.
  * Returns number of pixels whose values exceed 32000.0 or
  * are less then 50 sigma below the mean.
  * Inputs:
  * \param   in     pointer to minrtsFrameData   input frame
  */
int minrtsFrameDataSaturate (minrtsFrameData *in);

#endif /* end MINRTSFRAMEDATA_H */

